package java.awt;

import java.awt.geom.AffineTransform;
import java.awt.geom.Rectangle2D;
import java.awt.image.ColorModel;

final class RadialGradientPaintContext extends MultipleGradientPaintContext
{
    private boolean isSimpleFocus;
    private boolean isNonCyclic;
    private float radius;
    private float centerX;
    private float centerY;
    private float focusX;
    private float focusY;
    private float radiusSq;
    private float constA;
    private float constB;
    private float gDeltaDelta;
    private float trivial;
    private static final float SCALEBACK = 0.99f;
    private static final int SQRT_LUT_SIZE = 2048;
    private static float[] sqrtLut;
    
    RadialGradientPaintContext(RadialGradientPaint radialGradientPaint, ColorModel colorModel, Rectangle rectangle, Rectangle2D rectangle2D, AffineTransform affineTransform, RenderingHints renderingHints, float centerX, float centerY, float radius, float focusX, float focusY, float[] array, Color[] array2, MultipleGradientPaint.CycleMethod cycleMethod, MultipleGradientPaint.ColorSpaceType colorSpaceType) {
        super(radialGradientPaint, colorModel, rectangle, rectangle2D, affineTransform, renderingHints, array, array2, cycleMethod, colorSpaceType);
        this.isSimpleFocus = false;
        this.isNonCyclic = false;
        this.centerX = centerX;
        this.centerY = centerY;
        this.focusX = focusX;
        this.focusY = focusY;
        this.radius = radius;
        this.isSimpleFocus = (this.focusX == this.centerX && this.focusY == this.centerY);
        this.isNonCyclic = (cycleMethod == MultipleGradientPaint.CycleMethod.NO_CYCLE);
        this.radiusSq = this.radius * this.radius;
        float n = this.focusX - this.centerX;
        float n2 = this.focusY - this.centerY;
        double n3 = (double)(n * n + n2 * n2);
        if (n3 > (double)(this.radiusSq * 0.99f)) {
            float n4 = (float)Math.sqrt((double)(this.radiusSq * 0.99f) / n3);
            n *= n4;
            float n5 = n2 * n4;
            this.focusX = this.centerX + n;
            this.focusY = this.centerY + n5;
        }
        this.trivial = (float)Math.sqrt((double)(this.radiusSq - n * n));
        this.constA = this.a02 - this.centerX;
        this.constB = this.a12 - this.centerY;
        this.gDeltaDelta = 2.0f * (this.a00 * this.a00 + this.a10 * this.a10) / this.radiusSq;
    }
    
    @Override
    protected void fillRaster(int[] array, int n, int n2, int n3, int n4, int n5, int n6) {
        if (this.isSimpleFocus && this.isNonCyclic && this.isSimpleLookup) {
            this.simpleNonCyclicFillRaster(array, n, n2, n3, n4, n5, n6);
        }
        else {
            this.cyclicCircularGradientFillRaster(array, n, n2, n3, n4, n5, n6);
        }
    }
    
    private void simpleNonCyclicFillRaster(int[] array, int n, int n2, int n3, int n4, int n5, int n6) {
        float n7 = this.a00 * (float)n3 + this.a01 * (float)n4 + this.constA;
        float n8 = this.a10 * (float)n3 + this.a11 * (float)n4 + this.constB;
        float gDeltaDelta = this.gDeltaDelta;
        n2 += n5;
        int n9 = this.gradient[this.fastGradientArraySize];
        for (int i = 0; i < n6; n += n2, n7 += this.a01, n8 += this.a11, ++i) {
            float n10;
            float n11;
            int j;
            for (n10 = (n7 * n7 + n8 * n8) / this.radiusSq, n11 = 2.0f * (this.a00 * n7 + this.a10 * n8) / this.radiusSq + gDeltaDelta / 2.0f, j = 0; j < n5 && n10 >= 1.0f; n10 += n11, n11 += gDeltaDelta, ++j) {
                array[n + j] = n9;
            }
            while (j < n5 && n10 < 1.0f) {
                int n15;
                if (n10 <= 0.0f) {
                    n15 = 0;
                }
                else {
                    float n12 = n10 * 2048.0f;
                    int n13 = (int)n12;
                    float n14 = RadialGradientPaintContext.sqrtLut[n13];
                    n15 = (int)((n14 + (n12 - (float)n13) * (RadialGradientPaintContext.sqrtLut[n13 + 1] - n14)) * (float)this.fastGradientArraySize);
                }
                array[n + j] = this.gradient[n15];
                n10 += n11;
                n11 += gDeltaDelta;
                ++j;
            }
            while (j < n5) {
                array[n + j] = n9;
                ++j;
            }
        }
    }
    
    private void cyclicCircularGradientFillRaster(int[] array, int n, int n2, int n3, int n4, int n5, int n6) {
        double n7 = (double)(-this.radiusSq + this.centerX * this.centerX + this.centerY * this.centerY);
        float n8 = this.a00 * (float)n3 + this.a01 * (float)n4 + this.a02;
        float n9 = this.a10 * (float)n3 + this.a11 * (float)n4 + this.a12;
        float n10 = 2.0f * this.centerY;
        float n11 = -2.0f * this.centerX;
        for (int n12 = n5 + n2, i = 0; i < n6; n += n12, ++i) {
            float n13 = this.a01 * (float)i + n8;
            float n14 = this.a11 * (float)i + n9;
            double n20;
            double n21;
            float n22;
            float n23;
            float n24;
            float n25;
            float n26;
            float n27;
            float n28;
            for (int j = 0; j < n5; n22 = n13 - this.focusX, n23 = n22 * n22, n24 = n14 - this.focusY, n25 = n23 + n24 * n24, n26 = (float)n20 - this.focusX, n27 = n26 * n26, n28 = (float)n21 - this.focusY, array[n + j] = this.indexIntoGradientsArrays((float)Math.sqrt((double)(n25 / (n27 + n28 * n28)))), n13 += this.a00, n14 += this.a10, ++j) {
                if (n13 == this.focusX) {
                    n20 = (double)this.focusX;
                    n21 = (double)this.centerY + ((n14 > this.focusY) ? ((double)this.trivial) : ((double)(-this.trivial)));
                }
                else {
                    double n15 = (double)((n14 - this.focusY) / (n13 - this.focusX));
                    double n16 = (double)n14 - n15 * (double)n13;
                    double n17 = n15 * n15 + 1.0;
                    double n18 = (double)n11 + -2.0 * n15 * ((double)this.centerY - n16);
                    float n19 = (float)Math.sqrt(n18 * n18 - 4.0 * n17 * (n7 + n16 * (n16 - (double)n10)));
                    n20 = (-n18 + ((n13 < this.focusX) ? ((double)(-n19)) : ((double)n19))) / (2.0 * n17);
                    n21 = n15 * n20 + n16;
                }
            }
        }
    }
    
    static {
        RadialGradientPaintContext.sqrtLut = new float[2049];
        for (int i = 0; i < RadialGradientPaintContext.sqrtLut.length; ++i) {
            RadialGradientPaintContext.sqrtLut[i] = (float)Math.sqrt((double)((float)i / 2048.0f));
        }
    }
}
